@@ -45,16 +45,19 @@ import random
 import shout
 import urllib
 import mimetypes
-from threading import Thread
-from tools import *
 
+from threading import Thread
+from player import *
+from recorder import *
+from relay import *
+from streamer import *
+from tools import *
 
 class Station(Thread):
     """a DeeFuzzer shouting station thread"""
 
     id = 999999
     counter = 0
-    command = 'cat '
     delay = 0
     start_time = time.time()
 
@@ -74,26 +77,33 @@ class Station(Thread):
         self.ogg_quality = self.station['media']['ogg_quality']
         self.samplerate = self.station['media']['samplerate']
         self.voices = self.station['media']['voices']
+        self.m3u_playlist_file = []
+        if 'm3u' in self.station['media'].keys():
+            self.m3u_playlist_file = self.station['media']['m3u']
 
         # Server
-        self.short_name = self.station['infos']['short_name']
+        if 'mountpoint' in self.station['server'].keys():
+            self.mountpoint = self.station['server']['mountpoint']
+        elif 'short_name' in self.station['infos'].keys():
+            self.mountpoint = self.station['infos']['short_name']
+        else:
+            self.mountpoint = 'default'
+
+        self.short_name = self.mountpoint
 
         if 'type' in self.station['server']:
             self.type = self.station['server']['type'] #  'icecast' | 'stream-m'
-
-        if self.type == 'stream-m':
-            self.mount = '/publish/' + self.short_name
-        elif self.type == 'icecast':
-            self.mount = '/' + self.short_name
         else:
-            self.mount = '/' + self.short_name
+            self.type = 'icecast'
 
         if 'stream-m' in self.type:
             self.channel = HTTPStreamer()
-            self.channel.mount = self.mount
-        else:
+            self.channel.mount = '/publish/' + self.mountpoint
+        elif 'icecast' in self.type:
             self.channel = shout.Shout()
-            self.channel.mount = self.mount + '.' + self.media_format
+            self.channel.mount = '/' + self.mountpoint + '.' + self.media_format
+        else:
+            sys.exit('Not a compatible server type. Choose "stream-m" or "icecast".')
 
         self.channel.url = self.station['infos']['url']
         self.channel.name = self.station['infos']['name'] + ' : ' + self.channel.url
@@ -107,22 +117,25 @@ class Station(Thread):
         self.channel.public = int(self.station['server']['public'])
         self.channel.genre = self.station['infos']['genre']
         self.channel.description = self.station['infos']['description']
-        self.channel.audio_info = { 'bitrate': self.bitrate,
-                                    'samplerate': self.samplerate,
-                                    'quality': self.ogg_quality,
-                                    'channels': self.voices,}
+        self.channel.audio_info = { 'bitrate': str(self.bitrate),
+                                    'samplerate': str(self.samplerate),
+                                    'quality': str(self.ogg_quality),
+                                    'channels': str(self.voices),}
+
         self.server_url = 'http://' + self.channel.host + ':' + str(self.channel.port)
         self.channel_url = self.server_url + self.channel.mount
 
         # RSS
         self.rss_dir = self.station['rss']['dir']
         self.rss_enclosure = self.station['rss']['enclosure']
+
         if 'media_url' in self.station['rss']:
             self.rss_media_url = self.station['rss']['media_url']
             if self.rss_media_url[-1] != '/':
                 self.rss_media_url = self.rss_media_url + '/'
         else:
             self.rss_media_url = self.channel.url + '/media/'
+
         self.base_name = self.rss_dir + os.sep + self.short_name + '_' + self.channel.format
         self.rss_current_file = self.base_name + '_current.xml'
         self.rss_playlist_file = self.base_name + '_playlist.xml'
@@ -142,9 +155,27 @@ class Station(Thread):
             os.makedirs(self.metadata_dir)
 
         # The station's player
-        self.player = Player()
+        self.player = Player(self.type)
         self.player_mode = 0
 
+        # OSCing
+        self.osc_control_mode = 0
+        # mode = 0 means Off, mode = 1 means On
+        if 'control' in self.station:
+            self.osc_control_mode = int(self.station['control']['mode'])
+            if self.osc_control_mode:
+                self.osc_port = self.station['control']['port']
+                self.osc_controller = OSCController(self.osc_port)
+                # OSC paths and callbacks
+                self.osc_controller.add_method('/media/next', 'i', self.media_next_callback)
+                self.osc_controller.add_method('/media/relay', 'i', self.relay_callback)
+                self.osc_controller.add_method('/twitter', 'i', self.twitter_callback)
+                self.osc_controller.add_method('/jingles', 'i', self.jingles_callback)
+                self.osc_controller.add_method('/record', 'i', self.record_callback)
+                self.osc_controller.add_method('/player', 'i', self.player_callback)
+                self.osc_controller.add_method('/run', 'i', self.run_callback)
+                self.osc_controller.start()
+
         # Jingling between each media.
         # mode = 0 means Off, mode = 1 means On
         self.jingles_mode = 0
@@ -180,7 +211,7 @@ class Station(Thread):
             except:
                 pass
 
-            if self.twitter_mode == 1:
+            if self.twitter_mode:
                 self.twitter_callback('/twitter', [1])
 
         # Recording
@@ -189,121 +220,124 @@ class Station(Thread):
         if 'record' in self.station:
             self.record_mode = int(self.station['record']['mode'])
             self.record_dir = self.station['record']['dir']
-            if self.record_mode == 1:
+            if self.record_mode:
                 self.record_callback('/record', [1])
 
         # Running
         # mode = 0 means Off, mode = 1 means On
         self.run_mode = 1
 
-        # OSCing
-        self.osc_control_mode = 0
-        # mode = 0 means Off, mode = 1 means On
-        if 'control' in self.station:
-            self.osc_control_mode = int(self.station['control']['mode'])
-            self.osc_port = self.station['control']['port']
-            if self.osc_control_mode == 1:
-                self.osc_controller = OSCController(self.osc_port)
-                self.osc_controller.start()
-                # OSC paths and callbacks
-                self.osc_controller.add_method('/media/next', 'i', self.media_next_callback)
-                self.osc_controller.add_method('/media/relay', 'i', self.relay_callback)
-                self.osc_controller.add_method('/twitter', 'i', self.twitter_callback)
-                self.osc_controller.add_method('/jingles', 'i', self.jingles_callback)
-                self.osc_controller.add_method('/record', 'i', self.record_callback)
-                self.osc_controller.add_method('/player', 'i', self.player_callback)
-                self.osc_controller.add_method('/run', 'i', self.run_callback)
-
     def run_callback(self, path, value):
         value = value[0]
         self.run_mode = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
 
     def media_next_callback(self, path, value):
         value = value[0]
         self.next_media = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
 
     def relay_callback(self, path, value):
         value = value[0]
-        if value == 1:
+        if value:
             self.relay_mode = 1
             if self.type == 'icecast':
                 self.player.start_relay(self.relay_url)
-        elif value == 0:
+        else:
             self.relay_mode = 0
             if self.type == 'icecast':
                 self.player.stop_relay()
+
         self.id = 0
         self.next_media = 1
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
-        message = "Station " + self.channel_url + " : relaying : %s" % self.relay_url
+        message = "Station " + self.channel_url + \
+                    " : relaying : %s" % self.relay_url
         self.logger.write_info(message)
 
     def twitter_callback(self, path, value):
         value = value[0]
-        import tinyurl
         self.twitter = Twitter(self.twitter_key, self.twitter_secret)
         self.twitter_mode = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
-        self.m3u_tinyurl = tinyurl.create_one(self.channel.url + '/m3u/' + self.m3u.split(os.sep)[-1])
-        self.rss_tinyurl = tinyurl.create_one(self.channel.url + '/rss/' + self.rss_playlist_file.split(os.sep)[-1])
+        message = "Station " + self.channel_url + \
+                " : received OSC message '%s' with arguments '%d'" % (path, value)
+        self.m3u_url = self.channel.url + '/m3u/' + self.m3u.split(os.sep)[-1]
+        self.rss_url = self.channel.url + '/rss/' + self.rss_playlist_file.split(os.sep)[-1]
         self.logger.write_info(message)
 
     def jingles_callback(self, path, value):
         value = value[0]
-        if value == 1:
+        if value:
             self.jingles_list = self.get_jingles()
             self.jingles_length = len(self.jingles_list)
             self.jingle_id = 0
         self.jingles_mode = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
 
     def record_callback(self, path, value):
         value = value[0]
-        if value == 1:
+        if value:
             if not os.path.exists(self.record_dir):
                 os.makedirs(self.record_dir)
             self.rec_file = self.short_name.replace('/', '_') + '-' + \
-              datetime.datetime.now().strftime("%x-%X").replace('/', '_') + '.' + self.channel.format
+              datetime.datetime.now().strftime("%x-%X").replace('/', '_') + \
+                                                '.' + self.channel.format
             self.recorder = Recorder(self.record_dir)
             self.recorder.open(self.rec_file)
-        elif value == 0 and not self.type == 'stream-m':
-            self.recorder.close()
-            date = datetime.datetime.now().strftime("%Y")
-            if self.channel.format == 'mp3':
-                media = Mp3(self.record_dir + os.sep + self.rec_file)
-            if self.channel.format == 'ogg':
-                media = Ogg(self.record_dir + os.sep + self.rec_file)
-            media.metadata = {'artist': self.artist.encode('utf-8'),
-                                'title': self.title.encode('utf-8'),
-                                'album': self.short_name.encode('utf-8'),
-                                'genre': self.channel.genre.encode('utf-8'),
-                                'date' : date.encode('utf-8'),}
-            media.write_tags()
+        else:
+            try:
+                self.recorder.recording = False
+                self.recorder.close()
+            except:
+                pass
+            if self.type == 'icecast':
+                date = datetime.datetime.now().strftime("%Y")
+                if self.channel.format == 'mp3':
+                    media = Mp3(self.record_dir + os.sep + self.rec_file)
+                if self.channel.format == 'ogg':
+                    media = Ogg(self.record_dir + os.sep + self.rec_file)
+                media.metadata = {'artist': self.artist.encode('utf-8'),
+                                    'title': self.title.encode('utf-8'),
+                                    'album': self.short_name.encode('utf-8'),
+                                    'genre': self.channel.genre.encode('utf-8'),
+                                    'date' : date.encode('utf-8'),}
+                media.write_tags()
+
         self.record_mode = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
 
     def player_callback(self, path, value):
         value = value[0]
         self.player_mode = value
-        message = "Station " + self.channel_url + " : received OSC message '%s' with arguments '%d'" % (path, value)
+        message = "Station " + self.channel_url + \
+                    " : received OSC message '%s' with arguments '%d'" % (path, value)
         self.logger.write_info(message)
 
     def get_playlist(self):
         file_list = []
-        for root, dirs, files in os.walk(self.media_dir):
-            for file in files:
-                s = file.split('.')
-                ext = s[len(s)-1]
-                if ext.lower() == self.channel.format and not os.sep+'.' in file:
-                    file_list.append(root + os.sep + file)
-        file_list.sort()
+        if not self.m3u_playlist_file:
+            for root, dirs, files in os.walk(self.media_dir):
+                for file in files:
+                    s = file.split('.')
+                    ext = s[len(s)-1]
+                    if ext.lower() == self.channel.format and not os.sep+'.' in file:
+                        file_list.append(root + os.sep + file)
+            file_list.sort()
+        else:
+            f = open(self.m3u_playlist_file, 'r')
+            for path in f.readlines():
+                if '#' != path[0]:
+                    file_list.append(path[:-1])
         return file_list
 
     def get_jingles(self):
@@ -319,12 +353,12 @@ class Station(Thread):
 
     def get_next_media(self):
         # Init playlist
-        if self.lp != 0:
+        if self.lp:
             playlist = self.playlist
             new_playlist = self.get_playlist()
             lp_new = len(new_playlist)
 
-            if lp_new != self.lp or self.counter == 0:
+            if lp_new != self.lp or not self.counter:
                 self.id = 0
                 self.lp = lp_new
 
@@ -334,7 +368,7 @@ class Station(Thread):
                 new_tracks = new_playlist_set - playlist_set
                 self.new_tracks = list(new_tracks.copy())
 
-                if len(new_tracks) != 0:
+                if len(new_tracks):
                     new_tracks_objs = self.media_to_objs(self.new_tracks)
                     for media_obj in new_tracks_objs:
                         title = media_obj.metadata['title']
@@ -348,12 +382,13 @@ class Station(Thread):
                         if self.twitter_mode == 1:
                             artist_names = artist.split(' ')
                             artist_tags = ' #'.join(list(set(artist_names)-set(['&', '-'])))
-                            message = '#NEWTRACK ! %s #%s on #%s RSS: ' % (song.replace('_', ' '), artist_tags, self.short_name)
-                            message = message[:113] + self.rss_tinyurl
+                            message = '#NEWTRACK ! %s #%s on #%s RSS: ' % \
+                                     (song.replace('_', ' '), artist_tags, self.short_name)
+                            message = message[:113] + self.rss_url
                             self.update_twitter(message)
 
                 # Shake it, Fuzz it !
-                if self.shuffle_mode == 1:
+                if self.shuffle_mode:
                     random.shuffle(playlist)
 
                 # Play new tracks first
@@ -363,9 +398,10 @@ class Station(Thread):
 
                 self.logger.write_info('Station ' + self.channel_url + \
                                  ' : generating new playlist (' + str(self.lp) + ' tracks)')
-                self.update_rss(self.media_to_objs(self.playlist), self.rss_playlist_file, '(playlist)')
+                self.update_rss(self.media_to_objs(self.playlist),
+                                self.rss_playlist_file, '(playlist)')
 
-            if self.jingles_mode == 1 and (self.counter % 2) == 0 and not self.jingles_length == 0:
+            if self.jingles_mode and not (self.counter % 2) and self.jingles_length:
                 media = self.jingles_list[self.jingle_id]
                 self.jingle_id = (self.jingle_id + 1) % self.jingles_length
             else:
@@ -414,7 +450,8 @@ class Station(Thread):
             media_description_item = '<tr><td>%s:   </td><td><b>%s</b></td></tr>'
             for key in media.metadata.keys():
                 if media.metadata[key] != '':
-                    media_description += media_description_item % (key.capitalize(), media.metadata[key])
+                    media_description += media_description_item % (key.capitalize(),
+                                                                   media.metadata[key])
             media_description += '</table>'
 
             title = media.metadata['title']
@@ -469,17 +506,18 @@ class Station(Thread):
             pass
 
     def set_relay_mode(self):
-        self.prefix = '#nowplaying (relaying #LIVE)'
+        self.prefix = '#nowplaying #LIVE'
         self.title = self.channel.description.encode('utf-8')
         self.artist = self.relay_author.encode('utf-8')
         self.title = self.title.replace('_', ' ')
         self.artist = self.artist.replace('_', ' ')
         self.song = self.artist + ' : ' + self.title
+
         if self.type == 'stream-m':
             relay = URLReader(self.relay_url)
             self.channel.set_callback(relay.read_callback)
             if self.record_mode:
-             relay.set_recorder(self.recorder)
+                relay.set_recorder(self.recorder)
         else:
             self.stream = self.player.relay_read()
 
@@ -490,10 +528,12 @@ class Station(Thread):
         self.artist = self.current_media_obj[0].metadata['artist']
         self.title = self.title.replace('_', ' ')
         self.artist = self.artist.replace('_', ' ')
+
         if not (self.title or self.artist):
             song = str(self.current_media_obj[0].file_name)
         else:
             song = self.artist + ' : ' + self.title
+
         self.song = song.encode('utf-8')
         self.artist = self.artist.encode('utf-8')
         self.metadata_file = self.metadata_dir + os.sep + self.current_media_obj[0].file_name + '.xml'
@@ -502,6 +542,7 @@ class Station(Thread):
         self.logger.write_info('DeeFuzzing on %s :  id = %s, name = %s' \
             % (self.short_name, self.id, self.current_media_obj[0].file_name))
         self.player.set_media(self.media)
+
         if self.player_mode == 0:
             self.stream = self.player.file_read_slow()
         elif self.player_mode == 1:
@@ -516,7 +557,7 @@ class Station(Thread):
         message = '%s %s on #%s' % (self.prefix, self.song, self.short_name)
         tags = '#' + ' #'.join(self.twitter_tags)
         message = message + ' ' + tags
-        message = message[:108] + ' M3U: ' + self.m3u_tinyurl
+        message = message[:108] + ' M3U: ' + self.m3u_url
         self.update_twitter(message)
 
     def channel_open(self):
@@ -525,21 +566,25 @@ class Station(Thread):
 
     def ping_server(self):
         log = True
+
         while not self.server_ping:
             try:
                 self.q.get(1)
                 server = urllib.urlopen(self.server_url)
                 self.server_ping = True
-                self.logger.write_info('Station ' + self.channel_url + ' : channel available')
+                self.logger.write_info('Station ' + self.channel_url + \
+                                        ' : channel available')
                 self.q.task_done()
             except:
                 time.sleep(0.5)
                 if log:
-                    self.logger.write_error('Station ' + self.channel_url + ' : could not connect the channel' )
+                    self.logger.write_error('Station ' + self.channel_url + \
+                                        ' : could not connect the channel' )
                     log = False
                 self.q.task_done()
                 pass
 
+
     def run(self):
         self.q.get(1)
         self.ping_server()
@@ -571,13 +616,15 @@ class Station(Thread):
                     self.set_relay_mode()
                 elif os.path.exists(self.media) and not os.sep+'.' in self.media:
                     if self.lp == 0:
-                        self.logger.write_error('Station ' + self.channel_url + ' : has no media to stream !')
+                        self.logger.write_error('Station ' + self.channel_url + \
+                                                    ' : has no media to stream !')
                         break
                     self.set_read_mode()
                 self.q.task_done()
 
                 self.q.get(1)
-                if (not (self.jingles_mode and (self.counter % 2)) or self.relay_mode) and self.twitter_mode:
+                if (not (self.jingles_mode and (self.counter % 2)) or \
+                                    self.relay_mode) and self.twitter_mode:
                     try:
                         self.update_twitter_current()
                     except:
@@ -598,7 +645,9 @@ class Station(Thread):
                             self.recorder.write(self.chunk)
                             self.q.task_done()
                         except:
-                            self.logger.write_error('Station ' + self.channel_url + ' : could not write the buffer to the file')
+                            self.logger.write_error('Station ' + \
+                                            self.channel_url + \
+                                            ' : could not write the buffer to the file')
                             self.q.task_done()
                             continue
                     try:
@@ -607,15 +656,21 @@ class Station(Thread):
                         self.channel.sync()
                         self.q.task_done()
                     except:
-                        self.logger.write_error('Station ' + self.channel_url + ' : could not send the buffer')
+                        self.logger.write_error('Station ' + \
+                                                self.channel_url + \
+                                                ' : could not send the buffer')
                         self.q.task_done()
                         try:
                             self.q.get(1)
                             self.channel.close()
-                            self.logger.write_info('Station ' + self.channel_url + ' : channel closed')
+                            self.logger.write_info('Station ' + \
+                                                   self.channel_url + \
+                                                   ' : channel closed')
                             self.q.task_done()
                         except:
-                            self.logger.write_error('Station ' + self.channel_url + ' : could not close the channel')
+                            self.logger.write_error('Station ' + \
+                                                    self.channel_url + \
+                                                    ' : could not close the channel')
                             self.q.task_done()
                             continue
                         try:
@@ -623,10 +678,12 @@ class Station(Thread):
                             self.q.get(1)
                             self.channel_open()
                             self.channel.set_metadata({'song': self.song, 'charset': 'utf8',})
-                            self.logger.write_info('Station ' + self.channel_url + ' : channel restarted')
+                            self.logger.write_info('Station ' + self.channel_url + \
+                                                    ' : channel restarted')
                             self.q.task_done()
                         except:
-                            self.logger.write_error('Station ' + self.channel_url + ' : could not restart the channel')
+                            self.logger.write_error('Station ' + self.channel_url + \
+                                                ' : could not restart the channel')
                             self.q.task_done()
                             continue
                         continue
@@ -636,8 +693,3 @@ class Station(Thread):
 
             self.channel.close()
 
-
-
-
-
-
